#!/bin/bash
# Nathan Oyama <nathan.oyama@ibm.com> - Sat Jan  5 06:58:39 STD 2019

SCRIPT_DIR=$(dirname "$(test -L "$0" && readlink "$0" || echo "$0")")
TARGET_PATH="/opt/virgo/repository/ecx-usr" 
TEMP_PATH="/opt/virgo/repository/temp-properties"

# Ensure the script is run by root/sudo.
if [ "$EUID" -ne 0 ]; then
  echo "Permission denied. $(whoami), please use root or sudo. Aborting." >&2
  exit 1
fi

# Log
if [ -w /data/log/adminconsole/ ]; then
  LOGFILE="/data/log/adminconsole/properties-update.log"
else
  LOGFILE="/tmp/properties-update.log"
fi
env TZ=UTC date >> $LOGFILE
echo "$0 $@"    >> $LOGFILE 

# Set flag vars to empty.
HELP= QUIET=

while getopts :f:qh opt; do
  case $opt in
    h)  HELP=true
      ;;
    q)  QUIET=true
      ;;
    '?') echo "$0: invalid option -$OPTARG. Try $0 -h . Aborting." >&2 
      exit 1
      ;;
  esac
done
shift $((OPTIND - 1))

if [ $HELP ]; then
  echo "Usage: sudo bash ./isoprep_post.sh [OPTION]"
  echo "  Use this command after you have installed an update ISO file into IBM"
  echo "  Spectrum Protect Plus. This script will createe a backup of"
  echo "  all the properties files in $TARGET_PATH and store them in" 
  echo "  $TEMP_PATH , combine it with the other backup"
  echo "  you have created before the update, and finally,"
  echo "  overwrite properties files in $TARGET_PATH ."
  echo "Options:"
  echo "  -q : Suppress all normal output."
  echo "  -h : Show this help."
  exit 0
fi

if [ $QUIET ]; then
  2>/dev/null 1>/dev/null bash $0
  exit $?
fi

bash ${SCRIPT_DIR}/isoCompDuplicator.sh                    | tee -a $LOGFILE

VERSION="$(rpm -qa --queryformat "%{VERSION}-%{RELEASE}\n" spp-release)"
DAT_LIST="$(ls $TEMP_PATH/properties*.dat)"
echo "Available backup collections (.dat):" | tee -a $LOGFILE
echo "$DAT_LIST"

DATS_CURRENT_VER="$(grep -n "properties\.$VERSION" <<< "$DAT_LIST")"
DAT_CURRENT_VER_LATEST="$(tail -n 1 <<< "$DATS_CURRENT_VER" | cut -d ':' -f 2-)"

DAT_CURRENT_VER_FIRST_LN="$(head -n 1 <<< "$DATS_CURRENT_VER" \
  | cut -d ':' -f 1)"
DAT_PREVIOUS_VER_LATEST_LN="$((DAT_CURRENT_VER_FIRST_LN-1))"
DAT_PREVIOUS_VER_LATEST="$(\
  sed "${DAT_PREVIOUS_VER_LATEST_LN}q;d" <<< "$DAT_LIST")"

if [ ! -s "$DAT_PREVIOUS_VER_LATEST" ]; then
  echo "Invalid .dat for the latest previous version:"     | tee -a $LOGFILE >&2
  echo "$DAT_PREVIOUS_VER_LATEST does not exist or is "    | tee -a $LOGFILE >&2
  echo "empty. Please manually run isoCompMerger.sh ."     | tee -a $LOGFILE >&2
  echo "To see its usage, run"                             | tee -a $LOGFILE >&2
  echo "$ sudo bash ./isoCompMerger.sh -h"                 | tee -a $LOGFILE >&2
  echo "Aborting."                                         | tee -a $LOGFILE >&2
  exit 1
fi

if [ ! -s "$DAT_CURRENT_VER_LATEST" ]; then
  echo "Invalid .dat for the latest previous version:"     | tee -a $LOGFILE >&2
  echo "$DAT_CURRENT_VER_LATEST does not exist or is "     | tee -a $LOGFILE >&2
  echo "empty. Please manually run isoCompMerger.sh ."     | tee -a $LOGFILE >&2
  echo "To see its usage, use"                             | tee -a $LOGFILE >&2  
  echo "$ sudo bash ./isoCompMerger.sh -h"                 | tee -a $LOGFILE >&2
  echo "Aborting."                                         | tee -a $LOGFILE >&2
  exit 1
fi

bash ${SCRIPT_DIR}/isoCompMerger.sh \
  "$DAT_PREVIOUS_VER_LATEST" "$DAT_CURRENT_VER_LATEST"     | tee -a $LOGFILE

exit $?

