#!/usr/bin/python

####################################################################################################################
##### Script runs product udpates
#####
##### Syntax: update-manager.py --mode <checkupdate|update>
##### Example: update-manager.py --mode update
##### 
####################################################################################################################

from subprocess import call ,Popen, PIPE
import string
import random
import os, shutil, logging, sys, re, glob
from optparse import OptionParser, SUPPRESS_HELP 
import urllib , urllib2, json

CONST = None
FORWARD_SLASH="/"

class LessThenFilter(logging.Filter):
    def __init__(self, level):
	self._level = level
	logging.Filter.__init__(self)

    def filter(self, rec):
        return rec.levelno < self._level

def setup():
    
    global logger

    logger = logging.getLogger('update')
    logger.setLevel(logging.DEBUG)	

    # create a console handler
    cHandler = logging.StreamHandler(sys.stdout)
    cHandler.setLevel(logging.INFO)
    cHandler.setFormatter(logging.Formatter('%(message)s'))
    cHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(cHandler)


    # create a console handler for stderr
    ceHandler = logging.StreamHandler()
    ceHandler.setLevel(logging.ERROR)
    ceHandler.setFormatter(logging.Formatter('%(message)s'))
    #ceHandler.addFilter(LessThenFilter(logging.WARNING))
    logger.addHandler(ceHandler)

    # create a file handler
    fHandler = logging.FileHandler(options.log)
    fHandler.setLevel(logging.DEBUG)
    fHandler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s' , "%Y-%m-%d %H:%M:%S" ))
    logger.addHandler(fHandler)


def main():

        if options.yumclean: yumcleanall()

        if options.group is None : 
            group=repo.get_group_name()
        else: group=options.group

        if options.mode =='checkupdate':  check_update(group)

        if options.mode =='checkupgrade':  check_upgrade()

        if options.mode =='updateos':
            if options.package is not None:
                update_or_install_package(options.package)
            else:
                update_or_install_package('')

        if options.mode =='install': 
            if options.package is not None :
                install_package(options.package, True)

        if options.mode =='update':
            
            if options.package is not None : 
                update_or_install_package(options.package, True)
            
            elif options.group is None : 

                all_groups=group
                if is_update_available('endeavour-platform'): all_groups += ' endeavour-platform'
                # special handle for vsnap  , the group name is 'vsnap' 
                if is_update_available('vsnap'): 

                    update_or_install_package("vsnap-dist", True)

                    run_vsnap_dist_installer()


                update_or_install_package('spp-guestappvm', True)
                update_or_install_package('spp-nmt', True)

                update_group( all_groups , False)

            else:
                if  group == 'spp':
                    # fix update for disconnected from the 'spp' group spp-guestappvm
                    update_or_install_package('spp-guestappvm', True)
                    update_or_install_package('spp-nmt', True)

                update_group( group )

        if options.mode =='checkyumserver': checkyumserver()
        
        if options.mode =='updaterepourl': 
		if options.repourl is None:
                    logger.warning ("The '--updaterepourl' option expects url , pass with '--repourl'" )

                    failed_exit()
		else:
                    update_repo_url(options.repourl)

        if options.mode =='updatereposppv':

            if options.spp_version is None:
                logger.warning ("The '--updatereposppv' option expects version , pass with '--spp-version'" )

                failed_exit()
            else:
                update_repo_env_spp_version(options.spp_version)



def clean_exit():
        logger.info('Script completed successfully, Exiting!')
        sys.exit(0)

def failed_exit():
        #logger.debug('Script completed with failures ......')
        sys.exit(1)

#
# updates /etc/yum/vars/spp_version file
#
def update_repo_env_spp_version(version):
    logger.info('Update environmet variable for baseurl: ' + version)

    with open('/etc/yum/vars/spp_version' , 'w') as file:
        file.write(version)



def update_repo_url(repourl):
        logger.info('Repo url to update ' + repourl)

    	command = 'sed -i -e s#^baseurl=.*#baseurl=' + repourl + '#g ' + repo.get_config_file() 
    
	logger.info('Running command: ' + command)
    
	pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
	out, err = pobj.communicate()
    
	#logger.debug('Command Returncode : ' + str(pobj.returncode))

	if pobj.returncode == 0:
		if out:
			logger.info(out)            
			clean_exit()
        else:
		if out:
			logger.info(out)

		if err:
			logger.error(err)                
		
		failed_exit()


#
#  for internal use , does not exit
#    returns True if update is available, otherwise False
# 
def is_update_available(group):
    packages = get_packages_in_group( group )

    if  packages:
        s = ' '.join(packages)

        command = 'yum -q --disablerepo=* --enablerepo=' + repo.get_id() + ' check-update ' + s

        logger.info('Running command: ' + command)
        pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
        out, err = pobj.communicate()
        if pobj.returncode == 100: 
            return True
        else:
            logger.info('No updates available for packages ' + s)

    return False

#
# check_update()
#   arguments: group
#
def check_update(group):
 
    logger.info('Checking available update for group: ' + group )

    packages = get_packages_in_group( group )
    if not packages:
	logger.error('No packages in the group ' + group)
	failed_exit()

    s = ' '.join(packages)

    command = 'yum -q --disablerepo=* --enablerepo=' + repo.get_id() + '  check-update ' + s

    logger.info('Running command: ' + command)
    
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')
    out, err = pobj.communicate()
    
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 100:   # update is available
        if out:
            logger.info(out)            
            clean_exit()
    else:
        if out:
            logger.info(out)

        if err:
            logger.error(err)

        logger.error('Attempt to upgrade to an older or current version of IBM Spectrum Protect Plus is not permitted.  Upgrade is terminating.')

        failed_exit()

#
# checks update from current repo or looking into 'nvr' tag file ( upgrade ) 
#  returns list with one or two elements
# Example:
# [{ "version": "10.1.4", build: "380"}, { "version": "10.1.6", "build": ""}]
#
def check_upgrade():


    update_version = {}
    upgrade_version = {}
    update_upgrade_list = []

    update_version=repo.get_spp_version()

    if  update_version:  # if not empty and update is available
        update_upgrade_list.append(update_version)

    nsv=repo.get_next_supported_version()



    if  nsv :       
        upgrade_version["version"]=nsv
        upgrade_version["build"]=''
        update_upgrade_list.append(upgrade_version )


    update_upgrade_list_json=json.dumps(update_upgrade_list)

    #print (update_upgrade_list_json) 
    logger.info( update_upgrade_list_json )


#
# update_group()
#
def update_group(group, continue_flag=False):
    logger.info('Updating groups:  ' + group )

    command = 'yum --disablerepo=* --enablerepo=' + repo.get_id() + ' -y groupupdate ' + group

    logger.info('Running command: ' + command)
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

    while True:
        line = pobj.stdout.readline()
        logger.info(line)
        if line == '' and pobj.poll() != None:
            break
            
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag: clean_exit()
    else:

    	while True:
        	line = pobj.stderr.readline()
	        logger.error(line)
        	if line == '' and pobj.poll() != None:
	            break
        
        failed_exit()


#
#  check_installed_package()
#    returns true if package is installed
#
def check_installed_package(package):

    cmd=("rpm -q " + package )
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    stdout, stderr = p.communicate()

    #print ( 'debug  return code: ' + str(p.returncode) ) 
    
    if p.returncode == 0 : return True 
    else: return False

#
#  install command
#
def install_package( package, continue_flag=False):

    command = 'yum --disablerepo=* --enablerepo=' + repo.get_id() + ' -y install ' + package
    execute_yum_command(command,continue_flag)


#
# update_package()
#   update all if package is empty 
#
def update_or_install_package( package, continue_flag=False):

    if package=='':
        # only updates are allowed

        logger.info('Updating all packages from ' + repo.get_id() + ' repository' )
        command = 'yum --disablerepo=* --enablerepo=' + repo.get_id() + ' -y update '

    else:

        # package is not found, install the package
        if check_installed_package(package):
            logger.info('Updating package:  ' + package )
            command = 'yum --disablerepo=* --enablerepo=' + repo.get_id() + ' -y update ' + package

        else:
            logger.info('Installing package:  ' + package )
            command = 'yum --disablerepo=* --enablerepo=' + repo.get_id() + ' -y install ' + package

    execute_yum_command(command,continue_flag)


def execute_yum_command( command , continue_flag=False):

    logger.info('Running command: ' + command)
    pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

    while True:
        line = pobj.stdout.readline()
        logger.info(line)
        if line == '' and pobj.poll() != None:
            break
            
    #logger.debug('Command Returncode : ' + str(pobj.returncode))

    if pobj.returncode == 0:
        if not continue_flag: clean_exit()
    else:

    	while True:
        	line = pobj.stderr.readline()
	        logger.error(line)
        	if line == '' and pobj.poll() != None:
	            break
        
        failed_exit()

#
# run_vsnap_dist_installer()
#    runs /opt/vsnap-dist/vsnap-dist-*.run
#
def run_vsnap_dist_installer():

    list=glob.glob('/opt/vsnap-dist/vsnap-dist-*.run')

    if list: # installer exists
        installer = list[0]
        command = installer + '  -- nostart'
        pobj = Popen(command, bufsize=1, shell=True, stdout=PIPE, stderr=PIPE, cwd='/tmp')

        logger.info('Running command: ' + command)

        while True:
            line = pobj.stdout.readline()
            logger.info(line)
            if line == '' and pobj.poll() != None:
                break

        #logger.debug('Command Returncode : ' + str(pobj.returncode))

        if pobj.returncode == 0:
            logger.info('Successful installation of vsnap distribution')
        else:

            while True:
                    line = pobj.stderr.readline()
                    logger.error(line)
                    if line == '' and pobj.poll() != None:
                        break


#
# get_packages_in_group()
#
def get_packages_in_group(group):

    packages = []

    cmd = 'yum -q --disablerepo=* --enablerepo=' + repo.get_id() + ' groupinfo ' + group

    logger.info('Running command: ' + cmd)
    start_read=0
    p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
    with p.stdout:
        for line in iter(p.stdout.readline, b''):
            if line.startswith(" Mandatory Packages:"):
                start_read=1
                continue
            if line.startswith(" Installed Packages:"): continue

            if start_read: packages.append(line.strip().strip('=').strip('-').strip('+') )
    p.wait() # wait for the subprocess to exit

    return packages
    

def  checkyumserver():
    status=repo.get_http_server_code()
    if str(status) == '200': return 0     
    else:
        failed_exit() 

def yumcleanall():
    logger.info('Cleaning yum cache...')
    cmd=['yum','--enablerepo=*', 'clean', 'all']
    Popen(cmd).wait()
 
#       
# Represents yum repository
#
class Repo:
    'Class for yumrepository'

    def __init__(self,repo='localrepo'):

        self.group_name  = 'spp'

        self.id = repo
        cmd = 'yum repoinfo ' + repo

        p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
        stdout, stderr = p.communicate()

        for line in stdout.splitlines():
            if line.startswith("Repo-name    :"):  self.name    = line.split(':')[1]
            if line.startswith("Repo-status  :"):  self.status  = line.split(':')[1]
            if line.startswith("Repo-baseurl :"):  
                self.baseurl = line.split(' : ')[1]
                if not self.baseurl.endswith('/'): self.baseurl+='/'



        #print (self.baseurl )


 
    def get_id(self):
        return self.id
 
    def get_baseurl(self):
        return self.baseurl

    def get_group_name(self):
        return self.group_name

    def get_config_file(self):
	return self.config

    def get_http_server_code(self):

        if self.baseurl.startswith("http"):
            code=urllib.urlopen(self.baseurl).getcode()

            logger.debug('HTTP status code (' + self.baseurl + ')  : ' + str(code))

        else:
            # check internet connection anyway
            code=urllib.urlopen('http://www.google.com').getcode()
            logger.info('Local repository ' + self.id + ' wad detected , checking internet connectivity anyway: ' + str(code))

        return code

    #
    # reads 'nsv' file, supplied with image or updated manually
    #
    def get_next_supported_version(self):
        
        version=''
        target_url=self.baseurl + 'nsv'

        try:
            f = urllib2.urlopen(target_url)
            read_data=f.read()
            version=read_data.strip()
        except: 
            logger.warning ('Failed to open url ' + target_url + ' , or file does not exist')

            
        return version


    #
    # resolve version from release package 
    # yum check-update returns: 
    #  spp-release.noarch  <version>  <reponame>
    # 
    # retrns dictionary:
    #     {'version': '999.0.0', 'build': '1228'}
    def get_spp_version(self):
        
        release_package='spp-release'

        dict = {}

        try:

            cmd = 'yum -q --disablerepo=* --enablerepo=' + repo.get_id() + ' check-update  ' + release_package

            if options.verbosity > 1:
                logger.info('Running command: ' + cmd)

            p = Popen(cmd, stdout=PIPE, bufsize=1, shell=True, stderr=PIPE)
            stdout, stderr = p.communicate()
            
            m = re.match( r'.*' + release_package + '.\S+\s+(\S+).*', stdout , re.S)
            if(m):

                #logger.info('Version available on update: ' + m.group(1) )
                version,build=m.group(1).split('-')
                dict['version'] = version
                dict['build']   = build
                
            else:
                logger.warning('Can not detect version eligible for update :  ' + cmd)
                

        except Exception as e: 
            logger.warning(e)

        return dict



if __name__ == "__main__":

    version="1.8.2"
    parser = OptionParser()
    usage = "usage: %prog [options]"
    parser = OptionParser(usage=usage, version="%prog " + version)
    
    parser.add_option("-l","--log",
                      default='/data/log/adminconsole/update-manager.log',
                      dest='log',
                      help="Log file location"
                      " [ default: %default ]" )

    parser.add_option("--group",
                      dest='group',
                      help=SUPPRESS_HELP ) 

    parser.add_option("--package",
                      dest='package',
                      help=SUPPRESS_HELP ) 

    parser.add_option("-v", action="count", dest="verbosity", help=SUPPRESS_HELP)

    # undocumented option:  updatereposppv [update yum repo env vars for spp_version]
    parser.add_option("--mode",
                      help="supported mode: checkupdate, checkupgrade, update, updateos, checkyumserver, updaterepourl, install")


    parser.add_option("--yumclean", action="store_true", default=False,dest='yumclean', help=SUPPRESS_HELP )


    parser.add_option("--repourl",
		      dest='repourl',
		      help=SUPPRESS_HELP ) 

    parser.add_option("--spp-version",
		      dest='spp_version',
		      help=SUPPRESS_HELP ) 


    (options, args) = parser.parse_args()

    if options.mode is None:
        print ("A mandatory option 'mode' is missing\n")
        parser.print_help()
        failed_exit()

    setup()    


    if options.verbosity  > 1:
        logger.info('+----------------------- STARTING UPDATE MANAGER --------------------------------+')
        logger.info(os.path.realpath(__file__) + ' called with options:')
        logger.info(options)


    if options.mode =='updateos' and os.path.exists('/etc/yum.repos.d/local.repo'):
        repo=Repo('osupdate')

    elif  os.path.exists('/etc/yum.repos.d/local.repo'):
        repo=Repo('localrepo')

    elif options.mode =='updateos' and os.path.exists('/etc/yum.repos.d/osupdate.repo'):
        repo=Repo('osupdaterepo')

    elif os.path.exists('/etc/yum.repos.d/spp.repo'):
        repo=Repo('spprepo')
    else:
        print ('Can not detect yum repository')
        failed_exit()

    main()
